@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Mon Jan 18 04:01:46 MET 1993 on dec1.wu-wien.ac.at
+ * Creation: Tue Jun 15 04:25:32 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -99,7 +99,7 @@ char        **argv;
  * 	in: [char *]			# strings
  * 	const: sizeof(String)
  * 	const: 1
- * 	const: argc-1
+ * 	const: argc-2
  */
 
 int 
@@ -137,7 +137,7 @@ char        **argv;
           ptr ++; argv++;
           }
      }
-     AtLabelAxisAttachData(localVar1,localVar2,sizeof(String),1,argc-1);
+     AtLabelAxisAttachData(localVar1,localVar2,sizeof(String),1,argc-2);
      MMreplace(MMgetAttribList(localVar1),XrmStringToQuark("labelAxisAttachData"),(char*)localVar2,XtFree);
 
      DBUG_RETURN (TCL_OK);
@@ -312,7 +312,7 @@ char    **argv;
               convError("barPlotAttachData","in array", argv[count],"x/y");
               return;
               }
-          }
+	 }
 
      AtXYLinePlotAttachData(w,
 			    (XtPointer)xarray, AtFloat, sizeof(float),
@@ -375,6 +375,7 @@ Initialize_AtGen()
 #ifdef PLOTTER
  createWidgetCommand("axis",atAxisWidgetClass,NULL);
  createWidgetCommand("barPlot",atBarPlotWidgetClass,NULL);
+ createWidgetCommand("labelAxis",atLabelAxisWidgetClass,NULL);
  createWidgetCommand("linePlot",atLinePlotWidgetClass,NULL);
  createWidgetCommand("plotter",atPlotterWidgetClass,NULL);
  createWidgetCommand("textPlot",atTextPlotWidgetClass,NULL);
