@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Sat Oct  3 22:26:43 MET 1992 on dec1.wu-wien.ac.at
+ * Creation: Sun Jun 27 23:42:03 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -25,15 +25,17 @@
 #include "wafe.h"
 
 /*   this is also -*- C -*- */
+/*
+#define TUNNELDEBUG
+ */
 
-#include "rdd.h"
 
 char              *communicationHandlerVarname = NULL;
 char              *communicationHandlerCommand = NULL;
 int                communicationHandlerLength = 0;
 int                communicationCount = 0;
 
-char               inputBuffer[BUF_SIZE+1];
+char               inputBuffer[BUF_SIZE+2];
 int                leftBytes = 0;      
 
 
@@ -43,20 +45,33 @@ XtPointer   client_data;          /* PromptMode True oder False  */
 int         *fid;
 XtInputId   *xid;
     {
+
     char   *bufferPtr = inputBuffer + leftBytes;
+/*
     char   *firstPtr = inputBuffer;
+ */
+    char   *firstPtr = bufferPtr;
     char   *secondPtr = inputBuffer;
     int     readBytes;
     Boolean overFlowFlag = True;
 
     DBUG_ENTER("getInput");
 
+
+/*fprintf(stderr, "reading inputBuffer=%p, bufptr=%p, left=%d, can read=%d\n",
+	   inputBuffer, bufferPtr, leftBytes, BUF_SIZE-leftBytes);
+ */
+
+
     if ((readBytes = read(*fid, bufferPtr, BUF_SIZE - leftBytes)) == -1)
 	perror("Wafe: Communication error: ");
 
+/*fprintf(stderr, "read inputBuffer= %p, bufptr=%p, readBytes=%d, size=%d\n",
+	    inputBuffer, bufferPtr, readBytes, leftBytes+readBytes);
+ */
+    
     *(bufferPtr + readBytes) = '\0';
-    leftBytes = 0;
-
+   
     DBUG_PRINT("socketAll", 
                ("BufferContents are:\n>>%s<<", bufferPtr));     
 
@@ -73,43 +88,107 @@ XtInputId   *xid;
 
 	if (*firstPtr == '\n')
 	    {
+	    leftBytes = 0;
+	    overFlowFlag = False;
 	    *firstPtr = '\0';     /* One tcl-command is ready */
 	    
-	    overFlowFlag = False;
-
 	    DBUG_PRINT("socket", 
 		       ("Parsed Command >%s<", secondPtr));
-
-	    if (client_data)      /* Standard Mode - prompt character is checked */
-		{               
+	    
+	    if (client_data)   /* Standard Mode, checking magic chars */
+		{
 		if (*secondPtr == promptChar) 
 		    {
-		    if(Tcl_Eval(interpreter,secondPtr+1,0,NULL) == TCL_ERROR)
-			fprintf(stderr, "%s\n", interpreter->result);
+		    ++secondPtr;
+
+		    if (*secondPtr == '\\' || 
+                        *secondPtr == '+' || 
+                        *secondPtr == '=')
+                        {
+			char* namePtr = secondPtr + 1;
+			char* valPtr = namePtr;
+/*
+		        fprintf(stderr,
+                                "first=%p, second=%p\n",firstPtr,secondPtr);
+ */
+ 	                for(;valPtr<firstPtr && *valPtr != ' '; valPtr++);
+
+	                if (valPtr != namePtr)  /* we got a variable name */
+			    {
+			    if (valPtr != firstPtr) *valPtr++ = '\0';
+/*
+		        fprintf(stderr,
+                                "name=%p, value=%p\n",namePtr,valPtr);
+ */
+		            switch (*secondPtr) 
+				{
+			    case '\\':
+				if (valPtr != firstPtr)
+				    Tcl_SetVar(interpreter, namePtr, valPtr, 
+					     TCL_APPEND_VALUE|TCL_GLOBAL_ONLY);
+
+				Tcl_SetVar(interpreter, namePtr, "\n", 
+					   TCL_APPEND_VALUE|TCL_GLOBAL_ONLY);
+				DBUG_PRINT("assign", 
+				    ("appending to variable <%s> value <%s>nl",
+				      namePtr,valPtr));     
+				break;
+		    
+			    case '+':
+				if (valPtr != firstPtr) 
+				    {
+				    Tcl_SetVar(interpreter, namePtr, valPtr, 
+					TCL_APPEND_VALUE|TCL_GLOBAL_ONLY);
+
+				    DBUG_PRINT("assign", 
+					       ("appending to <%s> value <%s>",
+						namePtr,valPtr));     
+				    }
+				break;
+
+			    case '=':
+				Tcl_SetVar(interpreter, namePtr, valPtr, 
+					   TCL_GLOBAL_ONLY);
+				DBUG_PRINT("assign", 
+				    ("setting variable <%s> value <%s>",
+				      namePtr,valPtr));     
+				}
+		            }
+	                else  /* empty varname */
+			    fprintf(stdout, "%s\n", secondPtr-1);
+			}
+		    else /* no assignment character, evaluate the command */
+			if(Tcl_Eval(interpreter,secondPtr,0,NULL) == TCL_ERROR)
+			    fprintf(stderr, "%s\n", interpreter->result);
 		    }
-		else 
+		else   /* no prompt character */
 		    {
 		    if (tclHandlers[STDOUT])
 			{                    
 			Tcl_SetVar(interpreter, "STDOUT", secondPtr, 
 				   TCL_GLOBAL_ONLY);
 
-			if (Tcl_Eval(interpreter, tclHandlers[STDOUT], 
+			if (Tcl_Eval(interpreter, tclHandlers[STDOUT],
 				     0, NULL) == TCL_ERROR)
 			    fprintf(stderr, "%s\n", interpreter->result);
 			}
 		    else
-			fprintf(stdout, "%s\n", secondPtr);	 
+			fprintf(stdout, "%s\n", secondPtr);
 		    }
-		} 
+		}
 	    else                  /* Don't look for Prompt character */  
 		{                 
 		if (Tcl_Eval(interpreter, secondPtr, 0, NULL) == TCL_ERROR)
 		    fprintf(stderr, "%s\n", interpreter->result);
 		}
 	    
+	    /* we have processed something ending with \n */
 	    firstPtr++;
-	    
+	    /*
+	       fprintf(stderr,"one valid line processed %p-%p (%d)\n",
+	               secondPtr, firstPtr, firstPtr-secondPtr);
+	     */
+	    	    
 	    if (*firstPtr == '\0')      
 		{
 		DBUG_VOID_RETURN;
@@ -117,11 +196,23 @@ XtInputId   *xid;
 	    else
 		secondPtr = firstPtr;                  
 	    }
+	else
+	    {
+
+/*   fprintf(stderr, "BUFFER without nl leftBytes(%d)+readBytes(%d)=%d < %d\n",
+           leftBytes,readBytes,leftBytes+readBytes,BUF_SIZE);
+ */
+	    if (leftBytes + readBytes < BUF_SIZE) 
+		{
+		leftBytes = readBytes;
+		overFlowFlag = False;
+		}
+	    }
 	} while (*firstPtr != '\0');
 
     if (overFlowFlag)
 	{
-	fprintf(stderr, "Wafe(main): Uncomplete command or communication buffer overflow, >%s<\n", secondPtr);
+	fprintf(stderr, "Wafe(main): Incomplete command or communication buffer overflow, <%s>\n", secondPtr);
 	exit(-1);
 	}
 
@@ -129,10 +220,16 @@ XtInputId   *xid;
       * it is not terminated with a \n => do buffering 
       */
 
-    DBUG_PRINT("socket", ("uncomplete command : %s", secondPtr)); 
-
+    DBUG_PRINT("socket", ("incomplete command: <%s>, length=%d", 
+			  secondPtr,firstPtr-secondPtr)); 
+/*
     strcpy(inputBuffer, secondPtr);
-    leftBytes = strlen(secondPtr);
+    leftBytes = strlen(inputBuffer);
+ */
+
+    leftBytes = firstPtr-secondPtr;
+    if (inputBuffer != secondPtr) 
+	memcpy(inputBuffer, secondPtr,leftBytes);
 
     DBUG_VOID_RETURN;
     }
@@ -149,9 +246,13 @@ XtInputId   *xid;
 
     DBUG_ENTER("getInputFromStdin");
 
+/*fprintf(stderr,"HERE I AM readin STDIN expecting to read something\n");*/
+
     if ((readBytes = read(*fid, stdinBuffer, STDIN_BUF_SIZE)) == -1)
 	perror("getInput");
 
+/*fprintf(stderr,"HERE I AM readin STDIN, got %d bytes!\n", readBytes);*/
+
     if (!readBytes)
 	{
 	DBUG_PRINT("handler",
@@ -159,6 +260,8 @@ XtInputId   *xid;
 	DBUG_VOID_RETURN;
 	}
 
+/* fprintf(stderr,"HERE I AM readin STDIN, string=<%s>\n", stdinBuffer);*/
+
     if (tclHandlers[STDIN])
 	{
 	Tcl_SetVar(interpreter, "STDIN", stdinBuffer, TCL_GLOBAL_ONLY);
@@ -185,10 +288,12 @@ XtInputId   *xid;
 
     DBUG_ENTER("stderrHandlerProc");
 
+/*fprintf(stderr,"HERE I AM readin STDERR expecting to read something\n");*/
+
     if ((readBytes = read(*fid, stderrBuffer, STDERR_BUF_SIZE)) == -1)
 	perror("stderrHandlerProc");
 
-    *(stderrBuffer + readBytes) = '\0';
+/*fprintf(stderr,"HERE I AM readin STDERR, got %d bytes!\n", readBytes);*/
 
     if (!readBytes)
 	{
@@ -197,6 +302,10 @@ XtInputId   *xid;
 	DBUG_VOID_RETURN;
 	}
 
+    *(stderrBuffer + readBytes) = '\0';
+
+/*fprintf(stderr,"HERE I AM readin STDERR, string=<%s>\n", stderrBuffer);*/
+
     if (tclHandlers[STDERR])
 	{
 	Tcl_SetVar(interpreter, "STDERR", stderrBuffer, TCL_GLOBAL_ONLY);
@@ -296,61 +405,6 @@ XtInputId   *xid;
     DBUG_VOID_RETURN;
      }
 
-
-#ifdef NEVER
-int
-com_addTimeOut(clientData, comInterpreter, argc, argv)
-ClientData    clientData;
-Tcl_Interp   *comInterpreter;
-int           argc;
-char        **argv;
-    {
-    unsigned long   timeoutVal;
-     
-    DBUG_ENTER("addTimeOut");
-
-    if (argc != 3)
-	{
-	fprintf(stderr, "Wafe(addTimeOut): Wrong # of Args");
-	DBUG_RETURN(TCL_ERROR);
-	}
-
-    sscanf(argv[1],"%lu", &timeoutVal);
-    (void)XtAppAddTimeOut(appContext, timeoutVal, timerProc, 
-			  (XtPointer)XtNewString(argv[2]));
-
-    DBUG_RETURN(TCL_OK);
-    }
-#endif      
-
-#ifdef NEVER
-int
-com_getChannel(clientData, comInterpreter, argc, argv)
-ClientData    clientData;
-Tcl_Interp   *comInterpreter;
-int           argc;
-char        **argv;
-    {
-     int        i;
-     char       buffer[20];       
-
-     DBUG_ENTER("getChannel");
-     
-     if (argc != 1)
-          {
-          fprintf(stderr, "getChannel: Wrong # of args");
-          DBUG_RETURN (TCL_ERROR);
-	  }
-
-     sprintf(buffer, "%d", comClient);
-
-     Tcl_SetResult(comInterpreter, buffer, TCL_VOLATILE);
-     DBUG_RETURN(TCL_OK);
-     }
-#endif
-
-
-
 void
 setCommunicationVariable(varName,length,command)
 String   varName;
@@ -523,45 +577,6 @@ int handlerType;
      
 
 
-#define echo(x) {write(toClient,x,strlen(x)); if (count+1 != argc) write(toClient," ",1);}
-
-/* 
- * void
- * echo
- * 	callmultiple
- * 	in: String			# strings 
- * 	post: write(toClient, "\n",1);
- */
-
-int 
-com_echo(clientData, comInterpreter, argc, argv)
-ClientData    clientData;
-Tcl_Interp   *comInterpreter;
-int           argc;
-char        **argv;
-     {
-     String localVar1;
-     int count;
-
-     DBUG_ENTER("echo");  
-
-     if (argc < 2) 
-	 {
-	 argcError("echo","at least ",1,argc);
-	 DBUG_RETURN (TCL_ERROR);
-         }
-
-     for (count = 1; count < argc; count++)
-         {
-         localVar1 = argv[count];
-         echo(localVar1);
-          }
-     write(toClient, "\n",1);
-
-     DBUG_RETURN (TCL_OK);
-     }     
-
-
 #include <time.h>
 #define ref(a) &(a)
 
@@ -604,6 +619,45 @@ char        **argv;
      }     
 
 
+#define echo(x) {write(toClient,x,strlen(x)); if (count+1 != argc) write(toClient," ",1);}
+
+/* 
+ * void
+ * echo
+ * 	callmultiple
+ * 	in: String			# strings 
+ * 	post: write(toClient, "\n",1);
+ */
+
+int 
+com_echo(clientData, comInterpreter, argc, argv)
+ClientData    clientData;
+Tcl_Interp   *comInterpreter;
+int           argc;
+char        **argv;
+     {
+     String localVar1;
+     int count;
+
+     DBUG_ENTER("echo");  
+
+     if (argc < 2) 
+	 {
+	 argcError("echo","at least ",1,argc);
+	 DBUG_RETURN (TCL_ERROR);
+         }
+
+     for (count = 1; count < argc; count++)
+         {
+         localVar1 = argv[count];
+         echo(localVar1);
+          }
+     write(toClient, "\n",1);
+
+     DBUG_RETURN (TCL_OK);
+     }     
+
+
 #define getChannel() comClient
 
 /* 
@@ -677,7 +731,7 @@ char        **argv;
 /* 
  * void
  * doregister vulgo register
- * 	in: (int)handlerType		# stdin | stdout| stderr | xioerr
+ * 	in: (int)handlerType		# stdin | stdout | stderr | xioerr
  * 	in: char *			# tcl-command
  */
 
@@ -717,7 +771,7 @@ char        **argv;
         }
 
 
-    /* no need to assign to localVar2 = argv[2]  */ 
+    /* no need to assign  << localVar2 = argv[2] >>  */ 
 
      doregister(localVar1,argv[2]);
 
@@ -752,7 +806,7 @@ char        **argv;
 	 DBUG_RETURN (TCL_ERROR);
          }
 
-    /* no need to assign to localVar1 = argv[1]  */ 
+    /* no need to assign  << localVar1 = argv[1] >>  */ 
 
      if (!(sscanf(argv[2], "%d", &localVar2)))
           {
@@ -760,7 +814,7 @@ char        **argv;
           DBUG_RETURN (TCL_ERROR);
           }
 
-    /* no need to assign to localVar3 = argv[3]  */ 
+    /* no need to assign  << localVar3 = argv[3] >>  */ 
 
      setCommunicationVariable(XtNewString(argv[1]),localVar2,XtNewString(argv[3]));
 
@@ -771,7 +825,7 @@ char        **argv;
 /* 
  * void
  * unregister
- * 	in: (int)handlerType		# stdin | stdout| stderr | xioerr
+ * 	in: (int)handlerType		# stdin | stdout | stderr | xioerr
 
  */
 
@@ -824,8 +878,8 @@ void
 Initialize_MiscGen()
 {
 #ifdef MISC
- Tcl_CreateCommand(interpreter, "echo", com_echo, NULL, NULL);
  Tcl_CreateCommand(interpreter, "ctime", com_ctime, NULL, NULL);
+ Tcl_CreateCommand(interpreter, "echo", com_echo, NULL, NULL);
  Tcl_CreateCommand(interpreter, "getChannel", com_getChannel, NULL, NULL);
  Tcl_CreateCommand(interpreter, "quit", com_quit, NULL, NULL);
  Tcl_CreateCommand(interpreter, "register", com_register, NULL, NULL);
