@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Sun Feb  7 19:26:18 MET 1993 on dec1.wu-wien.ac.at
+ * Creation: Tue Jun 15 04:24:35 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -629,18 +629,27 @@ int           argc;
 char        **argv;
     {
     FILE    *fp;
-    int      index;
-    char   **list, **count, **oldList;
+    int      index, size=0;
+    char   **list, **count, **oldList,  *offset;
     char     buffer [MAX_ITEM_LENGTH] ;         
 
     DBUG_ENTER("listChange");
     
     if (!strcmp(argv[0], "Arg"))
-	{                      
-	list = count = (char **)XtMalloc(argc*sizeof(char *)); 
+	{
+        for (index = 1; index < argc; index ++) 
+	    {
+	    size += strlen(argv[index])+1;
+	    }
+	
+	list = count = (char **)XtMalloc((argc)*sizeof(char *)+size); 
+	offset = (char *)list + (argc)*sizeof(char *);
+
 	for (index = 1; index < argc; index++)
 	    {
-	    *count = XtNewString(argv[index]);
+	    *count = offset;
+	    strcpy(offset,argv[index]);
+	    offset += strlen(argv[index])+1;
 	    count++;
 	    }
 
@@ -649,6 +658,8 @@ char        **argv;
 
     if (!strcmp(argv[0], "File"))
 	{
+	char **tmpList, **saveStart;
+
 	if (argc != 2)
 	    {
 	    fprintf(stderr, "Wafe(listChange): Wrong # of Args");
@@ -662,41 +673,48 @@ char        **argv;
 	    DBUG_RETURN(TCL_ERROR);
 	    }
 
-	list = count = (char **)XtMalloc(10*sizeof(char *));
+	saveStart = tmpList = count = (char **)XtMalloc(10*sizeof(char *));
 	index = 0;
 
 	while (fgets(buffer, 200, fp))
 	    {
 	    *count = XtNewString(buffer);
+	    size += strlen(buffer)+1;
 	    *(*count + strlen(buffer)-1) = '\0';
 	    count++; index++;
 	    if (!(index % 10))
 		{
-		DBUG_PRINT("list", ("Realloc\n"));
-		list = (char**)XtRealloc((char *)list, (index + 10)
-					 * sizeof(char *));
-		count = list + index;
+		DBUG_PRINT("list", ("Realloc"));
+		saveStart = tmpList = (char**)XtRealloc((char *)tmpList,
+						(index + 10)* sizeof(char *));
+		count = tmpList + index;
 		}
 	    }
 	*count = NULL;
-	}
 
-    XtVaGetValues(w, XtNlist, &oldList, NULL);
-    XawListChange(w, list, nitems, longest, resize);
+	/* make it the list a single memory chunk to be 
+         * compatible with the rest 
+         */
+	list = count = (char **)XtMalloc((index+1)*sizeof(char *)+size); 
+	offset = (char *)list + (index+1)*sizeof(char *);
 
-    if (strcmp(widgetName,*oldList))
-	{
-	count = oldList;
-	while (*count)
+	while (*tmpList)
 	    {
-	    XtFree(*count);
+	    *count = offset;
+	    strcpy(offset,*tmpList);
+	    offset += strlen(*tmpList)+1;
+	    XtFree(*tmpList);
+	    tmpList++;
 	    count++;
 	    }
+	*count = NULL;
 
-	if (*oldList)
-	    XtFree(*oldList);
+        XtFree((char*)saveStart);
 	}
 
+    XawListChange(w, list, nitems, longest, resize);
+    MMreplace(MMgetAttribList(w), qPointer, (char *)list,XtFree);
+
     DBUG_RETURN(TCL_OK);
     }
 
@@ -2788,8 +2806,8 @@ char        **argv;
  * void
  * XawViewportSetLocation
  * 	in: Widget			# viewport Widget
- * 	in: Position			# xoffset 
- * 	in: Position			# yoffset
+ * 	in: double			# xoffset 
+ * 	in: double			# yoffset
 
  */
 #ifndef PRER5
@@ -2802,8 +2820,8 @@ int           argc;
 char        **argv;
      {
      Widget localVar1;
-     Position localVar2;
-     Position localVar3;
+     double localVar2;
+     double localVar3;
 
      DBUG_ENTER("viewportSetLocation");  
 
@@ -2819,15 +2837,15 @@ char        **argv;
           DBUG_RETURN (TCL_ERROR);
           }
 
-     if (!(sscanf(argv[2], "%hd", &localVar2)))
+     if (!(sscanf(argv[2], "%lf", &localVar2)))
           {
-          convError("viewportSetLocation","2",argv[2],"Position");
+          convError("viewportSetLocation","2",argv[2],"double");
           DBUG_RETURN (TCL_ERROR);
           }
 
-     if (!(sscanf(argv[3], "%hd", &localVar3)))
+     if (!(sscanf(argv[3], "%lf", &localVar3)))
           {
-          convError("viewportSetLocation","3",argv[3],"Position");
+          convError("viewportSetLocation","3",argv[3],"double");
           DBUG_RETURN (TCL_ERROR);
           }
 
