@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Mon Jan 18 04:01:31 MET 1993 on dec1.wu-wien.ac.at
+ * Creation: Sun Jun 27 23:42:24 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -207,7 +207,8 @@ char  *inString;
 /* Queries a XmString Value and translates it to a valid input string */
 
 char *
-xmString2String(compStr)
+xmString2String(buffer,compStr)
+char          * buffer;
 XmString      compStr;
     {
     XmStringContext    strContext;
@@ -217,7 +218,7 @@ XmString      compStr;
     XmStringDirection  dir, 
                        lastDir = XmSTRING_DIRECTION_L_TO_R;
     Boolean            sep;
-    char               *result, *ptr;
+    char               *returnString, *ptr;
     
     if (!XmStringInitContext(&strContext, compStr))
 	{
@@ -225,17 +226,28 @@ XmString      compStr;
 	return(NULL);
 	}
 
-    result = ptr = XtMalloc(XmStringLength(compStr)); /* ??? */
+    if (!buffer) 
+	{
+	ptr = returnString = XtMalloc(XmStringLength(compStr)); /* ??? */
+/*	fprintf(stderr, "buffer new allocated %p\n", ptr);*/
+	}
+    else
+	{
+/*	fprintf(stderr, "using provided buffer %p\n", buffer);*/
+	ptr = returnString = buffer;	
+	}
+
 
     while (XmStringGetNextSegment(strContext, &text, &charSet, &dir, &sep))
 	{
 	char *textPtr = text;
 
 	/* This means: If there are no changes in charSet or direction, just make a newline
-	            * but don't do it, if this is the first segment */
+	 * but don't do it, if this is the first segment 
+	 */
 	if ((!strcmp(charSet, lastCharSet)) 
 	    && (dir == lastDir) 
-	    && (result != ptr))
+	    && (returnString != ptr))
 	    *ptr++ = '\n'; 
 	
 	if (strcmp(charSet, lastCharSet)) 
@@ -271,9 +283,20 @@ XmString      compStr;
 	}
     *ptr = '\0';
     XmStringFreeContext(strContext);
-    return(result);
+    return(returnString);
     }
 
+void
+exmString2String(buffer,compStr)
+char          * buffer;
+XmString      compStr;
+    {
+    char *p = xmString2String(NULL,compStr);
+    estrcpy(buffer,p);
+    XtFree(p);
+    }
+
+
 
 char *
 xmStringTable2String(count,strTable)
@@ -285,19 +308,45 @@ XmStringTable strTable;
     char      *result;
     int        i;
 
-    if (!count && strTable)
+/*    fprintf(stderr, "string table to string: count = %d, stringTable = %p \n",count,strTable);
+ */
+/*
+    if (!count && strTable) 
+	{
+        fprintf(stderr, "counting...\n");
 	while(*ptr++)
+	    {
+	    fprintf(stderr, "count = %d\n",count);
 	    count++;
-
+	    }
+	}
+  */  
+    
     temp = (char **)XtMalloc(count*sizeof(char *));
-    for (ptr=strTable, cPtr=temp, i=count; i>0; ptr++, cPtr++, i--)
-	*cPtr = xmString2String(*ptr);
+    for (ptr=strTable, cPtr=temp, i=count; i>0; ptr++, cPtr++, i--) 
+	{
+/*	fprintf(stderr, "calling xmString2String\n"); */
+	*cPtr = xmString2String(NULL,*ptr);
+/*	fprintf(stderr, "it returned %p <%s>, ptr placed to %p\n",*cPtr,*cPtr, cPtr); */
+	}
+    
 
     result = Tcl_Merge(count, temp);
     
-    for (ptr = strTable, i=count; i>0; ptr++, i--)
+/*    for (ptr = strTable, i=count; i>0; ptr++, i--)
+	{
+	fprintf(stderr, "freeing %p, pointer at %p\n",*ptr, ptr);
 	XtFree(*ptr);
-    
+	}
+*/  
+
+    for (cPtr = temp, i=count; i>0; cPtr++, i--)
+	{
+/*	fprintf(stderr, "freeing %p, pointer at %p\n",*cPtr, cPtr);*/
+	XtFree(*cPtr);
+	}
+  
+    XtFree(temp);
     return(result); 
     }
 
@@ -317,7 +366,7 @@ XtPointer *converterData;
     {
     static char *result;
 
-    result = xmString2String((XmString)(fromVal->addr));
+    result = xmString2String(NULL,(XmString)(fromVal->addr));
     toVal->addr = result;
     toVal->size = 1; /* if nobody else uses this converter it shouldn't matter anyway */
     return(True);
@@ -388,7 +437,7 @@ XtPointer *converter_data;
     toVal->size = sizeof(XmString **);
     toVal->addr = (caddr_t)&result;
 
-    MMreplace(&currentStrInfo, currentAttrib, (char *)&result, freeStringTab);
+    MMreplace(currentAttribList, currentAttrib, (char *)&result, freeStringTab);
 
     return True;
     }
@@ -416,7 +465,7 @@ XtPointer *converter_data;
     toVal->size = sizeof(XmString *);
     toVal->addr = (caddr_t)&result;
 
-    MMreplace(&currentStrInfo, currentAttrib, (char *)&result, XmStringFree);
+    MMreplace(currentAttribList, currentAttrib, (char *)&result, XmStringFree);
     return True;
     }
 
@@ -1320,6 +1369,7 @@ Initialize_XmGen()
 	(XtTypeConverter)CvtXmStringTableToString, 
 	NULL, 0, XtCacheNone, NULL);
  compStringInit();
+ XtAddConverter (XmRString, XmRUnitType, XmCvtStringToUnitType, NULL, 0);;
  Tcl_CreateCommand(interpreter, "mCascadeButtonGadgetHighlight", com_mCascadeButtonGadgetHighlight, NULL, NULL);
  Tcl_CreateCommand(interpreter, "mCascadeButtonHighlight", com_mCascadeButtonHighlight, NULL, NULL);
  Tcl_CreateCommand(interpreter, "mCommandAppendValue", com_mCommandAppendValue, NULL, NULL);
