@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Sat Feb 13 02:00:48 MET 1993 on dec1.wu-wien.ac.at
+ * Creation: Tue Jun 15 04:23:45 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -866,7 +866,7 @@ char        **argv;
 /* 
  * String
  * getClass
- * 	in: Widget				# widget
+ * 	in: Widget			# widget | None
  */
 
 int 
@@ -905,8 +905,8 @@ char        **argv;
 /* 
  * int
  * getResourceList
- * 	in: Widget				# widget
- * 	out: String *				# varname for output of reslist
+ * 	in: Widget			# widget
+ * 	out: String *			# varname for output of reslist
  * 	end: XtFree((String)localVar2);
  */
 
@@ -953,9 +953,9 @@ char        **argv;
 /* 
  * XrmQuark
  * getQTypeOfAttribute vulgo getTypeOfAttribute
- * 	in: XtClass(Widget)			# widget
+ * 	in: XtClass(Widget)		# widget
  * 	const: ParentWidget(localVar1)
- * 	in: XrmStringToQuark(String)		# attribute
+ * 	in: XrmStringToQuark(String)	# attribute
  */
 
 int 
@@ -1380,7 +1380,7 @@ char        **argv;
      }     
 
 
-#define mergeResources(localVar1,argc,argv) { XrmDatabase    dispDb, newDb = NULL; int            i; char         **v = argv; dispDb = XtDatabase(XtDisplay(localVar1)); for (i = 0; i < argc; i+=2) XrmPutStringResource(&newDb, v[i], v[i+1]); XrmMergeDatabases(newDb, &dispDb);}
+#define mergeResources(localVar1,argc,argv) { XrmDatabase    dispDb, newDb = NULL; int            i; char         **v = argv; dispDb = XtDatabase(XtDisplay(localVar1)); for (i = 0; i < argc; i+=2)  XrmPutStringResource(&newDb, v[i], v[i+1]); XrmMergeDatabases(newDb, &dispDb);}
 
 /* 
  * void 
@@ -2255,7 +2255,7 @@ char        **argv;
  * XStoreBuffer
  * 	in: XtDisplay(Widget)		# widget to determine display
  * 	in: String			# String
- * 	const: strlen(argv[1])
+ * 	const: strlen(argv[2])
  * 	in: int				# buffer number (1-7)
  */
 
@@ -2292,7 +2292,7 @@ char        **argv;
           DBUG_RETURN (TCL_ERROR);
           }
 
-     XStoreBuffer(XtDisplay(localVar1),argv[2],strlen(argv[1]),localVar3);
+     XStoreBuffer(XtDisplay(localVar1),argv[2],strlen(argv[2]),localVar3);
 
      DBUG_RETURN (TCL_OK);
      }     
@@ -2368,6 +2368,7 @@ char        **argv;
 #ifdef XPM
 #include <xpm.h>
 #include <X11/extensions/shape.h>
+#include <unistd.h>
 
  /* Create a linked list, which keeps track of all created pixmaps,
   * so that we can destroy Colorcells, if they are not longer needed.
@@ -2389,7 +2390,6 @@ typedef struct _PixmapInfo
 PIXMAPINFOPTR  pixmapListHead = NULL;
 
 
-
 void
 changePixmap(w,resource,fname)
 Widget     w;
@@ -2606,11 +2606,13 @@ XtPointer *converter_data;
     {
     static Pixmap    pixmap;               /* static for cvt magic */
     Pixmap           dummy;
-    char            *name;
+    String           name;
     Screen          *screen;
     int              screenNum;
+/*
     XrmDatabase      db;
-    String           fn;
+ */
+    String           fn = NULL;
     unsigned int     width, height;
     int              xhot, yhot;
     unsigned char   *data;
@@ -2619,9 +2621,9 @@ XtPointer *converter_data;
     PIXMAPINFOPTR    pptr, cptr;
     Boolean          toPixmap = *num_args>0;
 
-    pixmap = NULL;
-    name   = (char *)fromVal->addr;
-/*    
+    pixmap = (Pixmap) NULL;
+    name   = (String) fromVal->addr;
+/*
     fprintf(stderr,"i am converting <%s> to %s\n",   
 	  name, toPixmap ? "Pixmap" : "Bitmap");
  */
@@ -2649,22 +2651,37 @@ XtPointer *converter_data;
         return True;
         }
 
-    if (!toPixmap)
+    if (!toPixmap) 
+        {
 	pixmap = XmuLocateBitmapFile(screen, name,
 				     NULL, 0, NULL, NULL, NULL, NULL);
+        }
 
 #ifndef PRER5
 
     if (pixmap == None) 
         {
+/*
         db = XrmGetDatabase(dpy);
         XrmSetDatabase(dpy, XtScreenDatabase(screen));
-        fn = XtResolvePathname(dpy, "bitmaps", name, "", NULL, NULL, 0, NULL);
+ */
+        if (access(name, R_OK) == 0)
+            fn = name;
+
         if (!fn)
+            {
+            fn = XtResolvePathname(dpy, "bitmaps", name, "", NULL, NULL, 0, NULL);
+	    /* fprintf(stderr,"1 read bitmap from <%s>  <%s>\n",fn,name); */
+            }
+
+        if (!fn)
+            {
             fn = XtResolvePathname(dpy, "", name, ".xbm", NULL, NULL, 0, NULL);
-	/* fprintf(stderr,"read bitmap from <%s>  <%s>\n",fn,name); */
+	    /* fprintf(stderr,"2 read bitmap from <%s>  <%s>\n",fn,name); */
+            }
+/*
 	XrmSetDatabase(dpy, db);
-	/* fprintf(stderr,"read bitmap from <%s>  <%s>\n",fn,name); */
+ */
 	if (fn &&
 	    XmuReadBitmapDataFromFile(fn, &width, &height, &data,
 				      &xhot, &yhot) == BitmapSuccess) 
@@ -2674,7 +2691,7 @@ XtPointer *converter_data;
 				 (char *) data, width, height, 1, 0, 
 				 DefaultDepth(dpy, DefaultScreen(dpy)));
 	    XFree((char *)data);
-            XtFree(fn);
+            if (name!=fn) XtFree(fn);
 	    }
         }
 #endif
@@ -2702,12 +2719,8 @@ XtPointer *converter_data;
 	    return True;
 	    }
 	}
-#endif
     }
-
-
-
-
+#endif
 
 /* 
  * hint: XPM
@@ -2758,7 +2771,6 @@ char        **argv;
  * 	in: Widget			# widget
  * 	in: String			# resource
  * 	in: String			# pixmap file
-
  */
 #ifdef XPM
 
@@ -2798,6 +2810,46 @@ char        **argv;
 
 #endif
 
+/* 
+ * int (int)
+ * _name2Widget vulgo isWidget
+ * 	in: String			# widget name 
+ * 	const: False
+ */
+
+int 
+com_isWidget(clientData, comInterpreter, argc, argv)
+ClientData    clientData;
+Tcl_Interp   *comInterpreter;
+int           argc;
+char        **argv;
+     {
+     String localVar1;
+     int returnVar;
+     char conversionBuffer[100];
+
+     DBUG_ENTER("isWidget");  
+
+     if (argc != 2) 
+	 {
+	 argcError("isWidget","",1,argc);
+	 DBUG_RETURN (TCL_ERROR);
+         }
+
+    /* no need to assign  << localVar1 = argv[1] >>  */ 
+
+     returnVar = (int) _name2Widget(argv[1],False);
+
+     sprintf(conversionBuffer, "%d", returnVar);
+     Tcl_SetResult(comInterpreter, conversionBuffer, TCL_VOLATILE);
+
+     DBUG_RETURN (TCL_OK);
+     }     
+
+
+#ifdef LAYOUT
+#include "Layout.h"
+#endif
 #else
 #include <string.h>
 #endif
@@ -2810,8 +2862,14 @@ Initialize_XtGen()
  createWidgetCommand("overrideShell",overrideShellWidgetClass,XtCreatePopupShell);
  createWidgetCommand("topLevelShell",topLevelShellWidgetClass,XtCreatePopupShell);
  createWidgetCommand("transientShell",transientShellWidgetClass,XtCreatePopupShell);
+#ifdef LAYOUT
+ createWidgetCommand("layout",layoutWidgetClass,NULL);
+#endif
 #ifdef XPM
  strcat(packages,"XPM ");
+#endif
+#ifdef LAYOUT
+ strcat(packages,"LAYOUT ");
 #endif
  Tcl_CreateCommand(interpreter, "addTimeOut", com_addTimeOut, NULL, NULL);
  Tcl_CreateCommand(interpreter, "action", com_action, NULL, NULL);
@@ -2873,6 +2931,7 @@ Initialize_XtGen()
 #ifdef XPM
  Tcl_CreateCommand(interpreter, "changePixmap", com_changePixmap, NULL, NULL);
 #endif
+ Tcl_CreateCommand(interpreter, "isWidget", com_isWidget, NULL, NULL);
 
 #else
 #endif
