@@ -145,7 +145,7 @@ char       **argv;
 	    if (managed) XtManageChild(new);
 	    }
 
-	MMsetAttribList(new,currentStrInfo);
+	MMsetAttribList(new); /* set attriblist(new) = currentAttribList) */
 /* 
     the only place where we could hide the attribListptr in
     the widget structure is the name 
@@ -159,7 +159,7 @@ char       **argv;
 	     p = XtMalloc(dummy+sizeof(XtPointer));
 	     strcpy(p,new->core.name);
 	     new->core.name = p;
-	     *(p+dummy) = (char *)currentStrInfo;
+	     *(p+dummy) = (char *)*currentAttribList;
 	    }
 */
 
