@@ -13,7 +13,7 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Sun Jan 17 01:35:39 MET 1993 on dec1.wu-wien.ac.at
+ * Creation: Tue Jun 15 04:25:56 MET DST 1993 on dec4.wu-wien.ac.at
  * Author: genc
  * Version: 0.92
  */
@@ -33,7 +33,7 @@ void myStartAction (w, event, args, nargs)
 	String *args;
 	int *nargs;
 {
-	static Pixmap pixmap = NULL;
+	static Pixmap pixmap = (Pixmap) NULL;
 	Display *dpy = XtDisplay(w);
 	Dimension width, height;
 	Position  x,y, xRoot, yRoot;
@@ -77,9 +77,9 @@ rddWafeInit()
     {
     static XtActionsRec actions[] = 
 	{
-	"rddDropAction", rddDropAction,
+	"rddDropAction", (XtActionProc) rddDropAction,
 	"wStartAction", myStartAction,
-	"rddStartAction", rddStartAction,
+	"rddStartAction", (XtActionProc) rddStartAction,
 	NULL, NULL,
 	};
     XtAppAddActions (appContext, actions, XtNumber(actions));
