@@ -16,9 +16,8 @@
  *
  * Date: Mon, Apr 13 1992
  * Author: Stefan Nusser, Gustaf Neumann
- * Version: 0.93
+ * Version: 0.95
  */
-
 /*
  *                 WAFE.C 
  *
@@ -57,6 +56,7 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <ctype.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -124,14 +124,14 @@ char *comp;
 
 
 
-/***********************************************************************************
-  *  The following functions maintain the global lists of ResourceLists, one for 
-  *  normal widgets and one for constraint resources. They look up the
-  *  resources registered for a specified widget class and - if they are queried for
-  *  the first time - append the new resourceList.
-  *  In each appended resourceList, the string name of the ressource will be substituted
-  *  by a quark representation.
-  ***********************************************************************************/
+/*****************************************************************************
+ *  The following functions maintain the global lists of ResourceLists, 
+ *  one for normal widgets and one for constraint resources. They look up the
+ *  resources registered for a specified widget class and - if they are 
+ *  queried for the first time - append the new resourceList. In each 
+ *  appended resourceList, the string name of the ressource will be 
+ *  substituted by a quark representation.
+ *****************************************************************************/
 
 QTypeList
 resList2QTypeList(rList, rNum, addResPtr, addNum)
@@ -161,7 +161,9 @@ Cardinal   addNum;
 	ptr->qType = XrmStringToQuark(ptr->type);
 	ptr++;
 	}
+
     XtFree((char *)rList);
+
     return(returnList);
     }
 
@@ -179,7 +181,7 @@ QTypeList  *qList;
     Boolean     disposeAddResPtr = False;
     
 
-    if (resourceList)  /* there are allready some entries in the resource list */
+    if (resourceList)  /* there exist already some entries */
 	{
 	for (lptr = resourceList; 
 	     lptr != NULL; 
@@ -206,7 +208,7 @@ QTypeList  *qList;
 #ifdef ATHENA
     /* fetch ATHENA AsciiText's subpart resources here! */
 
-    if (wClass == asciiTextWidgetClass)      /* check asciiText's subresources  */
+    if (wClass == asciiTextWidgetClass)  /* check asciiText's subresources */
 	{
 	XtResourceList rList[2], rl;
 	Cardinal       rNum[2], *num;
@@ -260,7 +262,7 @@ QTypeList  *qList;
     RESINFOPTR  ptr, lptr;
     XtResourceList conListPtr;
 
-    if (constraintList)  /* there are allready some entries in the resource list */
+    if (constraintList)  /* there exist already some entries */
 	{
 	for (lptr = constraintList; lptr != NULL; ptr = lptr, lptr = lptr->next) 
 	    if (lptr->wClass == wClass)
@@ -287,12 +289,12 @@ QTypeList  *qList;
 
 
 /*****************************************************************************
- *    FUNCTION:  getQTypeOfAttribute 
+ *  FUNCTION:  getQTypeOfAttribute 
  *                                  
- *    Arguments: Widgetclass, parent widget and Quark of attribute to be queried.
- *    Returns:   Type of the attribute as a Quark or NULL.                     
+ *  Arguments: Widgetclass, parent widget and Quark of attribute to be queried.
+ *  Returns:   Type of the attribute as a Quark or NULL.                     
  *                                                                      
- *    Used by    getValues, convert 
+ *  Used by    getValues, convert 
  *
  * Vielleicht wre es besser, eine Struktur mit einem String und einem Quark
  * zu retournieren, um ein etwaiges Zurckumwandeln zu ersparen. 
@@ -321,7 +323,7 @@ XrmQuark        qAttribute;
 	if (qAttribute == resList->qName)
 	    return(resList->qType);
           
-    if (XtIsConstraint(parent))         /* check Constraints if there are any... */
+    if (XtIsConstraint(parent))    /* check Constraints if there are any... */
 	{
 	count = getConOfClass(XtClass(parent), &resList);
 
@@ -347,9 +349,10 @@ XtPointer      info;
     (*ptr)->next = NULL;
     (*ptr)->info = info;
 /*
-    fprintf(stderr,"add to end of widgetlist *ptr %p, widget= %p, tree=%p info <%s>\n",
-	    *ptr,(*ptr)->widget,widgetTrees,(char*)((*ptr)->info));
- */
+   fprintf(stderr,
+           "add to end of widgetlist *ptr %p, widget= %p, tree=%p info <%s>\n",
+	   *ptr,(*ptr)->widget,widgetTrees,(char*)((*ptr)->info));
+*/
     }
 
 /*
@@ -371,6 +374,7 @@ Widget         w;
 
     XtFree((*ptr)->info);  /* let us assume, info is always mallocated */
     tmpPtr = (*ptr)->next;     
+
     XtFree((char *)(*ptr));
     *ptr = tmpPtr;
     }
@@ -405,8 +409,9 @@ Widget w;
     ptrTmp = MMwidgetListHead;
     MMwidgetListHead = (WidgetListPtr)XtMalloc(sizeof(WidgetListItem));
 /*
-    fprintf(stderr,"MMgetAttribList: new widgetlist at %p\n",MMwidgetListHead);
- */
+    fprintf(stderr,"MMgetAttribList: new widgetlist at %p, returning %p\n",
+	    MMwidgetListHead,&(MMwidgetListHead->info));
+*/
     MMwidgetListHead->next = ptrTmp;
     MMwidgetListHead->widget = w;
     MMwidgetListHead->info = NULL;
@@ -414,24 +419,19 @@ Widget w;
     }
 
 
-MMsetAttribList(w,al)
+MMsetAttribList(w)
 Widget w; 
-MMattribListPtr al;
     {
     WidgetListPtr   *ptr, p, ptrTmp;
-/*
-    ptr = &MMwidgetListHead;
-    if (al == NULL) 
-	while ((*ptr)->info && (*ptr)->next)
-	    ptr = &((*ptr)->next);
-*/
+
     ptrTmp = MMwidgetListHead;
     MMwidgetListHead = (WidgetListPtr)XtMalloc(sizeof(WidgetListItem));
     MMwidgetListHead->next = ptrTmp;
     MMwidgetListHead->widget = w;
-    MMwidgetListHead->info = (XtPointer)al;
+    MMwidgetListHead->info = (XtPointer)*currentAttribList;
 /*
-    fprintf(stderr,"MMset: setting al %p for widget %p\n",al,w);
+    fprintf(stderr,"MMset: setting al %p for widget %p\n",
+	    *currentAttribList,w);
 */
     }
 
@@ -485,15 +485,22 @@ char            *value;
 freeProc         freeFunc;
     {
     MMattribListPtr   *ptr = al;
+
     while(*ptr) 
 	{
+/*
+  fprintf(stderr,"comparing desc %d (%s) and att %d (%s)\n",
+    descriptor,XrmQuarkToString(descriptor),
+    (*ptr)->attribute,XrmQuarkToString((*ptr)->attribute)
+  );
+*/
 	if ((*ptr)->attribute == descriptor) 
 	    {
 	    (*freeFunc)((*ptr)->value);
 /*
 	    fprintf(stderr,"MMreplace: freeing value %p for <%s> setting %p\n",
 		    (*ptr)->value,XrmQuarkToString((*ptr)->attribute),value);
- */
+*/
 	    (*ptr)->value = value;
 	    return;
 	    }
@@ -504,10 +511,15 @@ freeProc         freeFunc;
     (*ptr)->next = NULL;                    
     (*ptr)->attribute = descriptor;
 /*
-    fprintf(stderr,"MMreplace: new attriblistelement %p for %s in al %p, value=%p\n",
+    fprintf(stderr,
+            "MMreplace: new attriblistelement %p for %s in al %p, value=%p\n",
 	    *ptr,XrmQuarkToString((*ptr)->attribute),al,value);
- */
+*/
     (*ptr)->value = value;
+
+#ifdef WIDGETLISTTRACE
+   printAttributeList(*ptr);
+#endif
     }
      
 
@@ -533,7 +545,7 @@ Widget      w;
 
 	XtVaGetValues(w, XtNnumChildren, &numChildren,
 		      XtNchildren,    &children,    NULL);
- 
+
 	while(numChildren) 
 	    {
 /*
@@ -557,8 +569,8 @@ Widget      w;
     while (*ptr) 
 	{
 /*
-       fprintf(stderr,"garbage: widget to delete %p <%s>, actual widget %p <%s>\n",
-	       (*ptr)->widget,XtName((*ptr)->widget),XtName(w));
+   fprintf(stderr,"garbage: widget to delete %p <%s>, actual widget %p <%s>\n",
+   (*ptr)->widget,XtName((*ptr)->widget),w,XtName(w));
  */
 	if ((*ptr)->widget == w) 
 	    {
@@ -630,6 +642,7 @@ Widget        w;
     XrmQuark          Qtype,Qname;
     int               i;
     int               count = 0; 
+    static MMattribListPtr currentStrInfo;
 
     DBUG_ENTER("convert");
 
@@ -638,6 +651,7 @@ Widget        w;
 	*numArgs = 0;
 	*args =NULL;
 	currentStrInfo = Nil(MMattribList);
+        currentAttribList = &currentStrInfo;
 	DBUG_RETURN (1);
 	}
 
@@ -649,11 +663,24 @@ Widget        w;
     
     argPtr = *args = (ArgList)XtMalloc((argc/2)*sizeof(Arg));
 
-    currentStrInfo = w ? (*(MMgetAttribList(w))) : Nil(MMattribList);
+    if (w) 
+        {
+	currentAttribList = MMgetAttribList(w);
+	currentStrInfo = *currentAttribList;
+        }
+    else
+        {
+        currentStrInfo = Nil(MMattribList);
+        currentAttribList = &currentStrInfo;
+        }
+/*
+	currentStrInfo = w ? (*(MMgetAttribList(w))) : Nil(MMattribList);
+ */
 /*
     fprintf(stderr,"convert: widget %p, currentStrInfo = %p\n",
 	    w, currentStrInfo);
- */   
+ */
+ 
     for (pairNum = 1; pairNum <= argc/2; pairNum++)
 	{
 	Qname = XrmStringToQuark(*argv); 
@@ -681,10 +708,13 @@ Widget        w;
 		if (Qtype == qString) 
 		    {
 		    argPtr->value = (XtArgVal)XtNewString(output.addr);
-		    MMreplace(&currentStrInfo,currentAttrib,
+		    MMreplace(currentAttribList,currentAttrib,
 			      (char *)(argPtr->value),XtFree);
 		    }
 		else 
+		if (output.size == sizeof(int))
+		    argPtr->value = (XtArgVal) *(int *)output.addr;
+		else
 		if (output.size == sizeof(long))
 		    argPtr->value = (XtArgVal) *(long *)output.addr;
 		else
@@ -695,10 +725,11 @@ Widget        w;
 		    {
 		    /* argPtr->value = (XtArgVal) (double *) output.addr;*/
 		    /* we have to copy doubles to allow multiple doubles
-		       in one widget creation or setValue command */
+		     *  in one widget creation or setValue command 
+		     */
 		    argPtr->value = (XtArgVal)XtMalloc(sizeof(double));
 		    *(double *)(argPtr->value) = *(double *)(output.addr);
-		    MMreplace(&currentStrInfo,currentAttrib,
+		    MMreplace(currentAttribList,currentAttrib,
 			      (char *)(argPtr->value),XtFree);
 		    }
 		else 
@@ -740,8 +771,9 @@ Widget        w;
 
 
 Widget
-name2Widget(name)
+_name2Widget(name,complain)
 char  *name;
+Boolean complain;
     {
     Widget result;
     char  starname[300];
@@ -773,7 +805,7 @@ char  *name;
 	ptr = ptr->next;
 	}
 
-    fprintf(stderr, "Wafe(name2Widget): %s not found!\n", name);
+    if (complain) fprintf(stderr, "Wafe(name2Widget): %s not found!\n", name);
     return (NULL);
     }
 
@@ -862,6 +894,27 @@ WidgetList widgetArray;
     return(Tcl_Merge(num, strArray));
     }
           
+
+void 
+estrcpy(output, input)
+char * output;
+char * input;
+    {
+    char * p = input;
+
+    if (!input) return;
+
+/*    *output++ = '"';*/
+    while (*p) 
+	{
+	if (*p == '"' || *p == '$' || *p == '[' || *p == '\\' ) 
+	    *output++ = '\\';
+
+	*output++ = *p++;
+	}
+/*    *output++ = '"';*/
+    *output = *p;
+    }
                
    
 /*****************************************************************************
@@ -881,7 +934,7 @@ int   sig;
     {
     DBUG_ENTER ("signalHandler");
     DBUG_PRINT("signal", ("Signal %d received", sig));
-    if (sig != SIGCLD) 
+    if (sig != SIGCLD)  
 	{
 	fprintf(stderr, "Wafe(main): Signal %d received!\n", sig);
         kill(pid, sig);
@@ -1039,13 +1092,13 @@ char **env;
     int                    cargc = 0;
     char                 **cargv;    
     char                 **cargvCount;
-    char                   file[50];
     char                  *charPtr;
     char                  *execString;
-
     int                    sp[4];  
     int                    ep[4];  
     int                    cp[4];
+    char                   file[50];
+    char                   intBuffer[INT_AS_STRING];
 
     static XtActionsRec    tclAction[] = {
 	{"exec",  execActionProc},
@@ -1068,18 +1121,19 @@ char **env;
 
     strHead = &STRINFOroot;
     cargvCount = cargv = (char **)XtMalloc((argc * sizeof(char *)) + 1);
-
-     /* In standard mode, the class will be derived from the application's name,
-       * in file mode from the filename and in direct  mode it is defined to be Wafe
-       */
+    
+    /* In standard mode, the class will be derived from the application's 
+     * name, in file mode from the filename and in direct  mode it is 
+     * defined to be Wafe
+     */
 
     strcpy(appClass, argv[0]);
     application[0] = '\0';
 
-     /*
-      * Argument parsing loop - build new argument list without wafe's 
-      * own args
-      */ 
+    /*
+     * Argument parsing loop - build new argument list without wafe's 
+     * own args
+     */ 
 
     while (args < argc)
 	{
@@ -1095,14 +1149,14 @@ char **env;
 	        }                             
 	    else
                              /***  Prompt - Mode  ***/ 
-	    if (argv[args][2] == 'n')            /* Turn prompt mode off */
+	    if (argv[args][2] == 'n')
 	        {                              
                 promptMode = False;               
                 args++;
                 }
 	    else
                              /***  Version  ***/ 
-            if (argv[args][2] == 'v')            /* Print Version */
+            if (argv[args][2] == 'v')
 	        {                              
 		fprintf(stderr, "\nThis is Wafe, version %s\n\n", VERSION);
                 fprintf(stderr, "Copyright (C) 1992 by Gustaf Neumann, Stefan Nusser\n");
@@ -1112,36 +1166,37 @@ char **env;
                 }
 	    else
                              /***  Input - Mode  ***/
-	    if (argv[args][2] == 'i')            /* Enter input mode  */
+	    if (argv[args][2] == 'i')
 	        {                            
                 inputMode  = True;            
                 args++;
                 }
 	    else
                              /***  Execute - Mode  ***/
-	    if (argv[args][2] == 'e')            /* Enter execute mode  */
-	        {      
-		int     comLength = 0;
-
-		args++;
-                execMode  = True;     
-		fileMode = True;
+	    if (argv[args][2] == 'e')
+	        {
+		args ++;
+                execMode = True;     
+/*            	fileMode = True; */
 		execString = argv[args];
 
 		if (execString == NULL)
 		    {
-		    fprintf(stderr, "Wafe: Option +e needs an additional argument\n");
+		    fprintf(stderr, 
+			    "Wafe: Option --e needs an additional argument\n");
 		    exit(-1);
 		    }
 
                 strcpy(appClass, "Wafe");
+		args ++;
                 }
 	    else
                              /***  Direct - Mode  ***/
-            if (argv[args][2] == 'd')            /* Enter direct mode */
+            if (argv[args][2] == 'd')
 	        {                           
                 directMode = True;
-
+		frontendMode = False;
+		
                 toClient = fileno(stdout);
                 fromClient = fileno(stdin);
                 errorClient = -1;
@@ -1167,15 +1222,16 @@ char **env;
 	        }
             else
                              /***  File - Mode  ***/
-            if (argv[args][2] == 'f')        /* Enter file mode */
+            if (argv[args][2] == 'f')     /* Enter file mode */
                 {
-                if (argv[args][3] == 'i')    /* For convenience: --fi == --f --i */
+                if (argv[args][3] == 'i') /* For convenience: --fi == --f --i*/
 		    {
-                     inputMode = True;
-		     }
+		    inputMode = True;
+		    }
 
                 fileMode = True;
-                strcpy(file, argv[args+1]);   /* Next Arg = File to execute */
+		frontendMode = False;
+                strcpy(file, argv[args+1]);  /* Next Arg = File to execute */
                 strcpy(appClass, file);
 
                 args +=2;
@@ -1208,40 +1264,41 @@ char **env;
 	DBUG_PRINT("start",("cargv[%d] = %s\n", i, cargv[i]));
 
      /*
-       * If the program's name contains a path specification cut it off...
-       */
+      * If the program's name contains a path specification cut it off...
+      */
     
     if (charPtr = strrchr(appClass, '/'))
 	strcpy(appClass, ++charPtr);
 
-   /*
-     *  We will need this information when we execute the application process ...
+    /*
+     * We will need this information when we execute the application process...
      */
 
     if (!application[0]) 
 	strcpy(application, appClass); 
 
-     /*
-       * If the name contains a dot, which is not allowed in Xt's class names, substitute it
-       * with a dash (keep this in mind, it will possibly affect the name of the AD file!
-       */
+    /*
+     * If the name contains a dot, which is not allowed in Xt's class names, 
+     * substitute it with a dash (keep this in mind, it will possibly affect 
+     * the name of the AD file!
+     */
 
     while(charPtr = strrchr(appClass, '.'))
 	*charPtr = '-';
          
-     /* Just because argv[0] will appear in the titlebar of the 
-      * top level window - if we were in file mode the title would
-      * always be wafe (which is probably not desired).
-      */
-
+    /* Just because argv[0] will appear in the titlebar of the 
+     * top level window - if we were in file mode the title would
+     * always be wafe (which is probably not desired).
+     */
+    
 
     cargv[0] = XtNewString(appClass);
 
     /* 
-      *  Now create class: Capitalized X, second letter capitalized, 
-      *  rest remains unchanged. Under this name, the app-defaults file
-      *  will be searched! 
-      */
+     *  Now create class: Capitalized X, second letter capitalized, 
+     *  rest remains unchanged. Under this name, the app-defaults file
+     *  will be searched! 
+     */
 
     if ((appClass[0] == 'x') || (appClass[0] == 'X'))
 	{
@@ -1251,15 +1308,15 @@ char **env;
     else
 	appClass[0] = toupper(appClass[0]);
 
-     /* Must initialize here, so that Xt can strip off its arguments */
+    /* Must initialize here, so that Xt can strip off its arguments */
 
     topLevel = XtVaAppInitialize(&appContext, appClass, NULL, 0, 
 				 &cargc, cargv, NULL, NULL); 
 /*
     XtAppSetErrorHandler(appContext,IOErrorHandler);
  */
-    XSetIOErrorHandler(IOErrorHandler);
 
+    XSetIOErrorHandler(IOErrorHandler);
 
     /* the default widget tree is under topLevel */
     addToEndOfWidgetList(&widgetTrees,topLevel,"topLevel");
@@ -1271,7 +1328,10 @@ char **env;
 	DBUG_PRINT("start",("cargv[%d] = %s\n", i, cargv[i])); 
 
 
-    if ((!directMode) && (!fileMode))     /* => standard mode            */
+    if (frontendMode && !opt_p) 
+	frontendMode = (*application == 'x') || (*application == 'X');
+    
+    if (frontendMode)     /* => running as a frontend      */
 	{     
 
 	 /*
@@ -1285,12 +1345,12 @@ char **env;
 	    exit(-1);
 	    }
 /*
-printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
-*/
+ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
+ */
 
-	 /*
-          *   This pair is used to facilitate an additional communication channel.
-          */
+	/*
+	 * This pair is used to facilitate an additional communication channel.
+	 */
 
 	if (iSocketPair(cp,1) == -1)
 	    {
@@ -1299,7 +1359,7 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
 	    }
 /*
  printf("cp %d %d %d %d\n",cp[0],cp[1],cp[2],cp[3]);
-*/
+ */
 
 	comClient = cp[1];  /* Communication endpoint for client  */
 	comWafe = cp[0];    /* Communication endpoint for wafe    */
@@ -1314,12 +1374,13 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
 
 	if (iSocketPair(ep,1) == -1)
 	    {
-	    fprintf(stderr, "Wafe(main): Couldn't create socket for app's stderr\n");
+	    fprintf(stderr, 
+		    "Wafe(main): Couldn't create socket for app's stderr\n");
 	    exit(-1);
 	    }
 /*
- printf("ep %d %d %d %d\n",ep[0],ep[1],ep[2],ep[3]);
-*/
+   printf("ep %d %d %d %d\n",ep[0],ep[1],ep[2],ep[3]);
+ */
 	pid = fork();
     
 	if (!pid)                       /* Child-process               */
@@ -1327,25 +1388,29 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
 	    dup2(sp[1], fileno(stdout));
 	    dup2(sp[2], fileno(stdin));
 	    dup2(ep[1], fileno(stderr));
+/*
+	    fprintf(stderr,
+		    "closing %d %d %d %d in CHILD\n", 
+		    ep[0], ep[1], sp[1], sp[0]);
 
+	    fprintf(stderr,"after dup %d = %d (stdin)\n", 
+		    sp[1], fileno(stdin));
+	    fprintf(stderr,"after dup %d = %d (stdout)\n", 
+		    sp[2], fileno(stdout));
+	    fprintf(stderr,"after dup %d = %d (stderr)\n", 
+		    ep[1], fileno(stderr));
+*/
 	    close(ep[0]);              /* Those 2 are for ..          */
 	    close(ep[1]);              /* getting stderr-messages.    */
 
 	    close(sp[1]);              /* For regular communication ..*/
 	    close(sp[0]);              /* via stdin and stdout!       */
 
-	    if (!opt_p && (*application != 'x') && (*application != 'X'))
-		{
-		fprintf(stderr, "Wafe(main): Wrong File was started, aborting\n");
-		sleep(1);
-		exit(-1);
-		}
 
 	    cargv[0] = opt_p ? application : application+1;
-
 	    execvp(cargv[0], cargv);   /* execute the client-program  */  
    
-	    fprintf(stderr, "Wafe(main): Error in exec-call to <%s>, aborting\n",
+	    fprintf(stderr, "Wafe(main): Cannot execute <%s>, aborting\n",
 		    cargv[0]);  
 	    exit(-1);
 	    }
@@ -1366,16 +1431,20 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
            * 1) errorClient   is the client's diagnostic output,
            * 2) toClient      leads to client's stdin and
            * 3) fromClient    is client's stdout 
-           * 4) wafeCom       additional communication channel
+           * 4) comWafe       additional communication channel
            */
 
 	errorClient = ep[0];
 	toClient = sp[3];
 	fromClient = sp[0];
-
-          /* If one of the following signals is caught, wafe exits and 
-           * the same signal is sent to the application
-           */
+/*
+	fprintf(stderr,
+		"errorClient=%d, toClient=%d, fromClient=%d, comWafe=%d\n", 
+		errorClient, toClient, fromClient,comWafe);
+ */
+	/* If one of the following signals is caught, wafe exits and 
+	 * the same signal is sent to the application
+	 */
 
 	signal(SIGTERM, signalHandler);  
 	signal(SIGQUIT, signalHandler);
@@ -1412,35 +1481,26 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
 
     if (!fileMode)
 	{
-/*
- printf("registering inputhandler fromClient %d\n",fromClient);
-*/
 	XtAppAddInput(appContext, fromClient, (XtPointer)XtInputReadMask,
-		      getInput, (XtPointer)(int)promptMode); /* to make gcc happy */
-
+		      getInput, (XtPointer)(int)promptMode); 
 
 	if (!directMode)
 	    { 
-/*
- printf("registering inputhandler errorCLient %d\n",errorClient);
-*/
-	    XtAppAddInput(appContext, errorClient,
-			  (XtPointer)XtInputReadMask, stderrHandlerProc, 
-			  (XtPointer)NULL); 
-
-/*
- printf("registering inputhandler comWafe %d\n",comWafe);
-*/
+            XtAppAddInput(appContext, errorClient,
+                          (XtPointer)XtInputReadMask, stderrHandlerProc, 
+                          (XtPointer)NULL); 
 	    XtAppAddInput(appContext, comWafe, (XtPointer)XtInputReadMask,
 			  communicationHandlerProc, NULL); 
 	    }
-
 	}
 
       
-    if ((inputMode) && (!execMode))
+    if ((inputMode) && (!execMode)) 
+	{
 	XtAppAddInput(appContext, fileno(stdin), (XtPointer)XtInputReadMask,
 		      stdinHandlerProc, NULL); 
+	}
+    
 
     XtAppAddActions(appContext, tclAction, 1);
     XtVaGetApplicationResources(topLevel, &app_data, resources,
@@ -1452,40 +1512,47 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
     for (i=0; i<HANDLERMAX; tclHandlers[i]=NULL, i++); 
     packages[0] = '\0';    
 
-{  COM_PROTOTYPE(Tcl_ListCmd)
-   Tcl_CreateCommand(interpreter, "llist", Tcl_ListCmd, NULL, NULL); 
-}
+	{  
+	COM_PROTOTYPE(Tcl_ListCmd)
+	    Tcl_CreateCommand(interpreter, "llist", Tcl_ListCmd, NULL, NULL); 
+	}
 
   /*
-      * The following include files contain the prototypings as well as the
-      * Tcl_createCommand statements for the automagically gernerated functions
-      */
+   * The following include files contain the prototypings as well as the
+   * Tcl_createCommand statements for the automagically gernerated functions
+   */
 
     callbackQuarkInitialize();
+    Initialize_XmGen();
     Initialize_XtGen();
     Initialize_MiscGen();
     Initialize_XawGen();
     Initialize_XawR5Gen();
-    Initialize_XmGen();
     Initialize_XmGraphGen();
     Initialize_AtGen();
     Initialize_XcGen();
     Initialize_rddGen();
-   
+
 /*  using non-caching converter for names to widget to avoid 
  *  dangling references (thanx Christian Holzbaur)
  */
     XtSetTypeConverter(XtRString, XtRWidget, 
-        (XtTypeConverter)CvtStringToWidget, 
-	NULL, 0, XtCacheNone, NULL);
-
+		       (XtTypeConverter)CvtStringToWidget, 
+		       NULL, 0, XtCacheNone, NULL);
+    
     XtSetTypeConverter(XtRString, XtRCallback, 
-	(XtTypeConverter)CvtStringToCallback, 
-	NULL, 0, XtCacheNone, NULL);
+		       (XtTypeConverter)CvtStringToCallback, 
+		       NULL, 0, XtCacheNone, NULL);
 
     XtSetTypeConverter(XtRString, XtRPointer, 
-	(XtTypeConverter)CvtStringToPointer, 
-	NULL, 0, XtCacheNone, NULL);
+		       (XtTypeConverter)CvtStringToPointer, 
+		       NULL, 0, XtCacheNone, NULL);
+
+#ifndef PRER5
+    XtSetTypeConverter(XtRString, XtRCardinal, 
+		       (XtTypeConverter)XtCvtStringToInt, 
+		       NULL, 0, XtCacheNone, NULL);
+#endif
 
 #ifdef PRER5
     Tcl_SetVar(interpreter, "XVERSION", "R4", TCL_GLOBAL_ONLY);
@@ -1495,6 +1562,9 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
     Tcl_SetVar(interpreter, "WAFEVERSION", VERSION, TCL_GLOBAL_ONLY);
     Tcl_SetVar(interpreter, "PACKAGES", packages, TCL_GLOBAL_ONLY);
 
+    Tcl_SetVar(interpreter, "ARGV", Tcl_Merge(cargc, cargv), TCL_GLOBAL_ONLY);
+    sprintf(intBuffer, "%d", cargc);
+    Tcl_SetVar(interpreter, "ARGC",  intBuffer, TCL_GLOBAL_ONLY);
 
     if (execMode)
         {
@@ -1502,31 +1572,19 @@ printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
 	    fprintf(stderr, "Wafe(execute mode): %s\n", interpreter->result);
 	}
 
-    if ((fileMode) && (!execMode))
-        {
-	char    intBuffer[20];
-
-	Tcl_SetVar(interpreter, "ARGV", Tcl_Merge(cargc, cargv), 
-		   TCL_GLOBAL_ONLY);
-
-	sprintf(intBuffer, "%d", cargc);
-	Tcl_SetVar(interpreter, "ARGC",  intBuffer, TCL_GLOBAL_ONLY);
-
-	for (i = 0; env[i] != NULL; i++);
-          Tcl_SetVar(interpreter, "ENV", Tcl_Merge(i, env), TCL_GLOBAL_ONLY);
-
+    if (fileMode)
 	if (Tcl_EvalFile(interpreter, file) == TCL_ERROR)
 	    {
 	    fprintf(stderr, "Wafe(file evaluation): %s\n", 
 		    interpreter->result);
 	    exit(-1);
 	    }
-	}
 
-    if (strcmp(app_data.init_com, "noCom"))
+    if (strcmp(app_data.init_com, ""))
         {
-	fprintf(stderr, "Wafe: Start mit %s\n", app_data.init_com);
-	SEND(app_data.init_com);
+	DBUG_PRINT("start", ("Wafe: Start mit %s\n", app_data.init_com));
+	write(toClient, app_data.init_com, strlen(app_data.init_com));
+	write(toClient, "\n", 1);
 	}
 
 #ifdef RDD
