@@ -1,13 +1,10 @@
 #include <dbug.h>
 #include <tcl.h>
 
-#define SEND(x) write(toClient, x ,strlen(x)); write(toClient, "\n", 1)
 #define ParentWidget(w) ((w) != topLevel ? XtParent(w) : (w))
 #define Nil(Typ)        ((Typ *) 0)     /* Make Lint happy */
 
 
-
-
 #ifndef PRER5
 #define IFCONVERTANDSTORE(w, from,inp, to,outp) \
 	if(XtConvertAndStore(w, from, & inp, to, & outp))
@@ -32,10 +29,11 @@
 #define MAX_TYPE_LEN		30
 #define INT_ARRAY_IN_BYTES	4096
 
+
 #define BUF_SIZE		65536
 #define COMBUFSIZE		65536
-#define STDIN_BUF_SIZE		1024
-#define STDERR_BUF_SIZE		1024
+#define STDIN_BUF_SIZE		10240
+#define STDERR_BUF_SIZE		10240
 
 
 #define YES                    1
@@ -77,7 +75,9 @@ WAFE_EXTERN int            errorClient; /* For error messages of application */
 WAFE_EXTERN int INITIALIZE(directMode, False); /* Flags for the different Modes  */
 WAFE_EXTERN int INITIALIZE(fileMode, False);
 WAFE_EXTERN int INITIALIZE(inputMode, False);
+WAFE_EXTERN int INITIALIZE(frontendMode, True);
 WAFE_EXTERN char INITIALIZE(promptChar, '%');
+WAFE_EXTERN char INITIALIZE(varChar, '=');
 
 WAFE_EXTERN XrmQuark       currentAttrib;
 
@@ -120,7 +120,7 @@ typedef struct _STRINFO
 
 WAFE_EXTERN STRINFO STRINFOroot;
 WAFE_EXTERN STRINFOPTR strHead;
-WAFE_EXTERN MMattribListPtr currentStrInfo;
+WAFE_EXTERN MMattribListPtr *currentAttribList;
 WAFE_EXTERN WidgetListPtr MMwidgetListHead;
 WAFE_EXTERN WidgetListPtr widgetTrees;
 
@@ -130,9 +130,11 @@ typedef void (*freeProc)(
 #endif			 
 );
 
-WAFE_EXTERN Widget name2Widget(
+#define name2Widget(name) _name2Widget(name,True)
+
+WAFE_EXTERN Widget _name2Widget(
 #if NeedFunctionPrototypes
-     char *
+     char *, Boolean
 #endif
 );
 
@@ -148,6 +150,12 @@ WAFE_EXTERN char * intsToTcl(
 #endif
 );
 
+WAFE_EXTERN void estrcpy(
+#if NeedFunctionPrototypes
+     String, String 
+#endif
+);
+
 WAFE_EXTERN Widget name2Source(
 #if NeedFunctionPrototypes
      char *
@@ -301,7 +309,7 @@ static XtResource resources[] =
      sizeof(String),
      XtOffset(AppDataPtr, init_com),
      XtRImmediate,
-     "noCom",
+     "",
      };
 
 
